/***************************************************************************
                          ppmcdiag.c  -  description
                             -------------------
    begin                : Mon Oct 23 2000
    copyright            : (C) 2000 by Eric Keller
    email                : eek105
 ***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include "ppmc.h"
#include "extintf.h"
#include <asm/io.h>

unsigned int ppmc_base_addr=0x378;

#define CONTROLPORT ppmc_base_addr+2
#define DATAPORT ppmc_base_addr+4

/* ident tag */
static char ident[] = "$Id: ppmcdiag.c,v 3.0 2005/04/20 19:29:58 jmelson Exp $";


/*
  Be root to run it
  */

#define NUM_AXES 4
enum {POS = 1, LIMIT, HOME, INDEX, LATCH, PWM, SERVO,SERVOF,DIO,DIOCONT,BUS,COMM,DAC,INDEXRES};
static int done = 0;
static void quit(int sig)
{
  done = 1;
}




int main(int argc, char *argv[])
{
  int which = POS;
  int newline=0;
  double pos[NUM_AXES];
  double oldpos[NUM_AXES];
  int lim[NUM_AXES*2];
  int oldlim[NUM_AXES*2];
  int home[NUM_AXES];
  int oldhome[NUM_AXES];
  double dacval[NUM_AXES];
  int index;
  int latch;
  int axis,errors;
  double volts;
  int t,var;
  unsigned int a,b;
  int l;
  char buf[80];
  char *cptr,*cptr2;
  //printf("diags univpwm test program 4/20/2005\n");
  if (argc<3)
	{
	  printf("syntax: %s I/O Address in hex <test name>\n", argv[0]);
	  printf("test name       Function\n");
	  printf("bus     reports all boards by address, type and version\n");
	  printf("pos     continuously reports encoder position\n");
	  printf("pwm     tests pwm statically, in count units\n");
	  printf("limit   tests limit switch sensing\n");
	  printf("home    tests home switch sensing\n");
	  printf("index   reports sensing of encoder index channel\n");
	  printf("latch   tests encoder latch function\n");
	  printf("servo   simulates a servo loop, reports position on screen\n");
	  printf("servof  same as above, no screen report, faster\n");
	  printf("dio     test DIO board function interactively\n");
	  printf("diocontinuous  tests DIO input and output continuously\n");
	  printf("commtest  continuously tests encoder communication on IEEE-1284 bus\n");
	  printf("diotest   continuously tests DIO communication\n");
	  printf("indexres  tests reset of encoder count on index pulse\n");
	  printf("  exit all looping tests with ctrl/c\n");
	    exit(-1);   //the program was hard to write, it should be hard to use
	}
	if (argc > 2)
  {
    sscanf(argv[1],"%x",&ppmc_base_addr);
    printf("io addr = %x\n",ppmc_base_addr);
      if (!strcmp(argv[2], "pos"))
	    {
	  		which = POS;
			}
      else if (!strcmp(argv[2], "pwm"))
			{
	  		which = PWM;
			}
      else if (!strcmp(argv[2], "limit"))
			{
	  		which = LIMIT;
			}
      else if (!strcmp(argv[2], "home"))
			{
	  		which = HOME;
			}
      else if (!strcmp(argv[2], "servo"))
			{
	  		which = SERVO;
			}
      else if (!strcmp(argv[2], "servof"))
			{
	  		which = SERVOF;
			}
      else if (!strcmp(argv[2], "bus"))
			{
	  		which = BUS;
			}
      else if (!strcmp(argv[2], "diocontinuous"))
			{
	  		which = DIOCONT;
			}
      else if (!strcmp(argv[2], "commtest"))
			{
	  		which = COMM;
			}
      else if (!strcmp(argv[2], "dac"))
			{
	  		which = DAC;
			}
      else if (!strcmp(argv[2], "index"))
			{
	  		if (1 == sscanf(argv[3], "%d", &axis))
	    	{
	      	which = INDEX;
	    	}
	  		else
	    	{
	      	printf("syntax: %s {pos | dac | limit | home | index <axis> | latch <axis>}"
		       "| servo{f} | dio | bus\n", argv[0]);
	      	exit(-1);
	    	}
			}
      else if (!strcmp(argv[2], "indexres"))
			{
	  		if (1 == sscanf(argv[3], "%d", &axis))
	    	{
	      	which = INDEXRES;
	    	}
	  		else
	    	{
	      	printf("syntax: %s {pos | dac | limit | home | index <axis> | latch <axis>}"
		       "| servo{f} | dio | bus\n", argv[0]);
	      	exit(-1);
	    	}
			}
      else if (!strcmp(argv[1], "latch"))
			{
	  		if (1 == sscanf(argv[3], "%d", &axis))
	    	{
	      	which = LATCH;
	    	}
	  		else
	    	{
	      	printf("syntax: %s {pos | dac | limit | home | index <axis> | latch <axis>}\n", argv[0]);
	      	exit(-1);
	    	}
		}
      else if (!strcmp(argv[2], "dio"))
	{
	  which = DIO;
	}
     else
		{
	      printf("syntax: %s {limit | home | index <axis> | latch <axis>}\n", argv[0]);
	  		exit(-1);
		}
  } //end if argc>1


  signal(SIGINT, quit);

  ppmcMotInit(NULL);
  printf("parport addr 0x%x \n", ppmc_base_addr);

  if (which == POS)
  {
      while (! done)
			{
	  			newline = 0;
	  			ppmcEncoderReadAll(NUM_AXES, pos);
	  			for (t = 0; t < NUM_AXES; t++)
	    		{
	      		if(fabs(pos[t]-oldpos[t]) > 1000.0 && !newline)
						{
		  				newline=1;
		  				printf("\n");
						}
	      		oldpos[t] = pos[t];
	    		}
	  			for (t = 0; t < NUM_AXES; t++)
	    		{
	      		printf("%+15.3f\t", pos[t]);
	    		}
	  			printf("\r");
	  			fflush(stdout);
	  			usleep(10000);
			}
  }
  if (which == SERVO)
  {
      while (! done)
       {
	  newline = 0;
	  ppmcEncoderReadAll(NUM_AXES, pos);
	  for (t = 0; t < NUM_AXES; t++)
          {
          if(fabs(pos[t]-oldpos[t]) > 1000.0 && !newline)
	    {
	    newline=1;
	    printf("\n");
	    }
	  oldpos[t] = pos[t];
       }
       for (t = 0; t < NUM_AXES; t++)
         {
	   printf("%+12.1f ", pos[t]);
           dacval[t] = (pos[t] / 1000.0);
	 }
       printf("   ");
       for (t = 0; t < NUM_AXES; t++)
	 {
	   printf("%+10.2f ",dacval[t]);
	 }
       printf("\n");
       ppmcRateWriteAll(NUM_AXES,dacval);
       fflush(stdout);
       usleep(10000);
    }
  }
  if (which == SERVOF)
  {
      while (! done)
       {
	  newline = 0;
	  ppmcEncoderReadAll(NUM_AXES, pos);
          for (t = 0; t < NUM_AXES; t++)
         {
             dacval[t] = (pos[t] / 1000.0);
	 }
       ppmcRateWriteAll(NUM_AXES,dacval);
    }
  }
  else if (which == LIMIT)
  {
      while (! done)
			{
	  		newline=0;
	  		for (t = 0; t < NUM_AXES; t++)
	    	{
	      	ppmcMaxLimitSwitchRead(1, &lim[2*t]);
	      	if(oldlim[2*t] != lim[2*t] && !newline)
					{
		  			printf("\n");
		  			newline=1;
					}
	      	oldlim[2*t] = lim[2*t];
	      	ppmcMinLimitSwitchRead(1, &lim[2*t+1]);
	      	if(oldlim[2*t+1] != lim[2*t+1] && !newline)
					{
		  			printf("\n");
		  			newline=1;
					}
	      	oldlim[2*t+1] = lim[2*t+1];
	    	}
	  		if(newline)
	    	{
	      	for (t = 0; t < NUM_AXES; t++)
					{
		  			printf("%+d\t", lim[2*t]);
		  			printf("%+d\t", lim[2*t+1]);
					}
	      	printf("\n");
	      	fflush(stdout);
	    }
	  	usleep(10000);
		}
  }
  else if (which == HOME)
  {
      while (! done)
			{
	  		newline=0;
	  		for (t = 0; t < NUM_AXES; t++)
	    	{
	      	ppmcHomeSwitchRead(t, &home[t]);
	      	if(oldhome[t] != home[t] && !newline)
					{
		  			printf("\n");
		 				newline=1;
					}
	      	oldhome[t] = home[t];
	    	}
	  		if(newline)
	    	{
	      		for (t = 0; t < NUM_AXES; t++)
						{
		  				printf("%d\t", home[t]);
						}
	      		printf("\r");
	      		fflush(stdout);
	    	}
	  		usleep(10000);
		}
  }
  else if (which == INDEX)
  {
      if (axis < 0 || axis >= NUM_AXES)
			{
	  		printf("syntax: %s {limit | home | index <axis> | latch <axis>}\n", argv[0]);
	  		exit(1);
			}

      ppmcEncoderResetIndex(axis);
      while (! done)
			{
	  		ppmcEncoderReadLevel(axis, &index);
	  		printf("%d ", index);
	  		fflush(stdout);
	  		usleep(10000);
			}
  }
  else if (which == INDEXRES)
  {
      if (axis < 0 || axis >= NUM_AXES)
			{
	  		printf("syntax: %s {limit | home | index <axis> | latch <axis>}\n", argv[0]);
	  		exit(1);
			}
      SelWrt(ENCLOAD_0,0);             /* clear 3-byte preload register */
      WriteData(0);
      WriteData(0);
      
      //      ppmcEncoderResetIndex(axis);
      SelWrt(0x0d,(1<<axis));   // turn on reset by index bit
      printf("setting index set reg (0x05) to %x\n",(1<<axis));
      while (! done)
	{
	  newline = 0;
	  ppmcEncoderReadAll(NUM_AXES, pos);
	  for (t = 0; t < NUM_AXES; t++)
	    {
	      if(fabs(pos[t]-oldpos[t]) > 1000.0 && !newline)
		{
		  newline=1;
		  printf("\n");
		}
	      oldpos[t] = pos[t];
	    }
	  for (t = 0; t < NUM_AXES; t++)
	    {
	      printf("%+15.3f\t", pos[t]);
	    }
	  printf("\r");
	  fflush(stdout);
	  usleep(10000);
	}
  }
  else if (which == LATCH)
  {
      if (axis < 0 || axis >= NUM_AXES)
			{
	  		printf("syntax: %s {limit | home | index <axis> | latch <axis>}\n", argv[0]);
	  		exit(1);
			}

      ppmcEncoderResetIndex(axis);
      while (! done)
			{
				ppmcEncoderReadLatch(axis, &latch);
				printf("%d ", latch);
				fflush(stdout);
				usleep(10000);
			}
	}
  else if (which == PWM)
	{
	  printf("Axis? ");
	  fgets(buf,80,stdin);
	  axis = (int) strtol(buf,0,0);
	  while( !done)
	    {
	      printf("RATE(%d)",axis);
	      printf("cycle, pwm? ");
	      if(fgets(buf,80,stdin) == NULL)
		{
		  printf("got null in dac\n");
		  break;
		}
	      if(!isdigit(buf[0]) && buf[0] != '-')
		{
		  printf("non numeric volts , quitting\n");
		  break;
		}
	      cptr = buf;
	      while((*cptr >= '0' && *cptr <= '9')     // find end of string
		    || *cptr == '-' || *cptr == '.' || *cptr == 'E' || *cptr == ' ')
		{
		  cptr++;
		}
	      *cptr = 0;                // put null at end
	      printf("first char in buf = %x\n",*buf);
	      a = strtoul(buf,&cptr,10);
	      printf("first char in cptr = %x\n",*cptr);
	      l = strtol(cptr,0,10);
	      printf("cycle =%d,      pwm = %d\n",a,b);
	      a = 65535 - a;
	      b = 65535 - labs(l);
	      printf("cycle =%x,      pwm = %x\n",a,b);
	      ppmcDioWrite(2,128);     // turn off E-stop
	      SelWrt(0x1d,(a & 255));   // send Limit
	      WriteData(a/256);
	      SelWrt(0x10,(b & 255));  // send PWM
	      WriteData(b/256);
	      if (l < 0) 
		SelWrt(0x1c,3);        // enable minus output
	      else
		SelWrt(0x1c,2);        // enable output
	      SelWrt(0x1f,0x80);  // see if a write to estop reg messes up PWM
	    }
	}  //end if dac
  else if (which == DIO)
    {
      printf("dio \n");
      ppmcDioRead(0, &axis);
      printf("Digital In 0 = %d,  ",axis);
      ppmcDioRead(1, &axis);
      printf("Digital In 1 = %d,  ",axis);
      ppmcDioRead(2, &axis);
      printf("Estop In = %d\n",axis);

      printf("Enter 1 to do output");
      fgets(buf,80,stdin);
      axis = (int) strtol(buf,0,0);
      if (axis == 1)
	{
	  printf("value to output ?");
	  if(isdigit(buf[0]))
	    {
	      ppmcDioWrite(2,128);     // turn off E-stop
	      cptr = buf + 1;
	      axis = (int) strtol(cptr,0,0);
	      printf("sending %d\n",axis);
	      ppmcDioWrite(0,axis);
	    }
	}
    }
  else if (which == DAC)
    {
      printf("DAC test, enter value ");
      fgets(buf,80,stdin);
      axis = (int) strtol(buf,0,0);
      if(isdigit(buf[0]))
	{
	  ppmcDioWrite(2,128);     // turn off E-stop
	  SelWrt(15,axis);
	}
    }
  else if (which == DIOCONT)
    {
      var = 0;     // bloop variable
      t = 0;     //slow down counter
      ppmcDioWrite(0,128);  // get out of estop
      ppmcDioWrite(0,128);  // and fix timing race ?
      ppmcDioWrite(0,128);
      ppmcDioWrite(0,128);
      ppmcDioWrite(0,128);
      printf("  DIO 0 inputs        DIO 1 inputs\n");
      printf(" 7 6 5 4 3 2 1 0     7 6 5 4 3 2 1 0\n");
      while (! done)
	{
	  ppmcDioRead(0, &axis);
	  for (a = 8 ; a--; a >= 0) {
     	    printf("%2d",(axis>>a)&0x01);
	  }
	  printf("    ");
	  ppmcDioRead(1, &axis);
	  for (a=8; a--; a >= 0) {
	    printf("%2d",(axis>>a)&0x01);
	  }
	  t++;
	  if (t > 500 ) {
	    t=0;
	    var = var<<1;
	    if ((var&0x100) != 0) {
	      var = var & 0xFE;
	    } else {
	      var = var | 1; }
	    ppmcDioWrite(0,var | 128);    // write to output port
	    printf("  output %x",var);
	    
	  }
	  printf("\r");

	}
    }
  else if (which == BUS)
     {
       printf("Bus Map \n  Board Addr  Type         Ver.\n");
       for (a = 15; a<=255; a+=16)
	 {
	   SelAddr(a);
	   outb(0x24,CONTROLPORT);  // set EPP port to input mode
	   b = inb(DATAPORT);
	   switch (b>>4)
	     {
	       case 0 : printf("%4x        Unknown       %4x\n",(a>>4),b); break;
	       case 1 : printf("%4x        Encoder       %4x\n",(a>>4),(b & 15)); break;
	       case 2 : printf("%4x        DAC-16        %4x\n",(a>>4),(b & 15)); break;
	       case 3 : printf("%4x        DIO           %4x\n",(a>>4),(b & 15)); break;
	       case 4 : {
		 printf("%4x        Univ. Stepper %4x\n",(a>>4),(b & 15));
		 a+=16;      // skip next addr as univ stepper uses 2 blocks
		 break;
	       }
	       case 5 : {
		 printf("%4x        Univ. PWM     %4x\n",(a>>4),(b & 15));
		 a+=16;      // skip next addr as univ pwm uses 2 blocks
		 break;
	       }
	       case 15 : printf("%4x        No Board      %4x\n",(a>>4),b); break;
	       default : printf("%4x        Unknown       %4x\n",(a>>4),b); break;

	     }
	 }
     }
  else if (which == COMM)
    {
      t =0;
      errors=0;
      while (! done) {
	t++;
	if ((t%1000) == 0)
	  printf(" %d test cycles, %d errors\n",t,errors);
	// aughhh !! big goof in decode of latstb in univstep!
	SelWrt(ENCCTRL_0,0xF0);  // select load pos counter and enable
	SelWrt(ENCCTRL_0,0xF0);  // and hold state for a while
	home[0] = rand() & 0xff;
	home[1] = rand() & 0xff;
	home[2] = rand() & 0xff;
	SelWrt(ENCCTRL_0,0xF0);
	SelWrt(ENCCTRL_0,0xF0);
	SelWrt(ENCCTRL_0,0xF0);
	SelWrt(ENCCTRL_0,0xF0);
	SelWrt(ENCLOAD_0,home[0]);     // load data into holding register
	WriteData(home[1]);
	WriteData(home[2]);
	SelWrt(ENCCTRL_0,0);  // and turn the load pos counter off
	//	ppmcEncoderReadAll(NUM_AXES, pos);
	EncoderLatch();  // latch data into storage registers
	SelAddr(ENCCNT0);   // some delay before real read
	SelAddr(ENCCNT0);
	SelAddr(ENCCNT0);
	outb(0x24,CONTROLPORT);    // set EPP to input mode
	for (a = 0; a < NUM_AXES; a++)
	  {
	    oldhome[0]=ReadData();
	    oldhome[1]=ReadData();
	    oldhome[2]=ReadData();
	    if ((home[0] != oldhome[0]) || (home[1] != oldhome[1]) || (home[2] != oldhome[2])) 
	      {
		errors++;
		printf(" test cycle %d, axis %d was / should be \n",t,a);
		printf(" %2x %2x %2x / %2x %2x %2x \n",oldhome[0],oldhome[1],oldhome[2],home[0],home[1],home[2]);
	      }
	  }	
      }
    }
  //ppmcAioQuit();
  //ppmcDioQuit();
  extMotQuit();

  return 0;
}


